package javax.rtgedition;

/***********************************************
 *  RTG, University of Pennsylvania
  * Author:	Dave Arney	11/07/01
  *                    Woongsup Kim
  **********************************************/

public class RationalTime extends RelativeTime
{
    int frequency;
    
    public RationalTime(int frequency)
    {
        try
        {
           new RationalTime(frequency,(long)1000,0);
        }
        catch(Exception e)
        {
            // Shouldn't get here!!!
        }
        
    }
    
    public RationalTime(int frequency, long millis, int nanos)
        throws IllegalArgumentException
    {
        this.frequency = frequency;
	this.millis = millis;
	this.nanos = nanos;
        if((millis/frequency <= 0 && nanos/frequency <= 0)
            || frequency <= 0 || millis < 0 || nanos < 0)
        {
            throw new IllegalArgumentException();
        }
    }
    
    public RationalTime(int frequency, RelativeTime interval)
        throws IllegalArgumentException
    {
        this(frequency,interval.millis,interval.nanos);
    }

    public AbsoluteTime absolute(Clock clock, AbsoluteTime destination)
    {
        destination = clock.getTime();
	destination.add(millis, nanos);
	return destination;
    }
	
    public void addIntervalTo(AbsoluteTime destination)
    {
	destination.add(this.millis/frequency,this.nanos/frequency,destination);
    }

    public int getFrequency()
    {
        return this.frequency;
    }
	
    public RelativeTime getInterarrivalTime(RelativeTime dest)
    {
	dest.set(this.millis/this.frequency,this.nanos/this.frequency);
	return dest;
    }
    
    public void set(long millis, int nanos) 
        throws IllegalArgumentException
    {
        millis = millis+nanos/NANOS_TO_MILLIS;
        nanos  = nanos%NANOS_TO_MILLIS;
        
        if((millis < 0 || nanos < 0) || (millis/this.frequency <= 0 && nanos/this.frequency <= 0))
        {
            throw new IllegalArgumentException();
        }
        
	this.millis = millis;
        this.nanos  = nanos;
    }

    /*
     * NOTE! POSSIBLE ERROR IN THE SPEC! We need to throw an IllegalArgumentException
     * if the argument causes the time interval to drop below 0!!!!
     *
     * @throws ArithmeticException if frequency <= 0
     * @throws IllegalArgumentException if this frequency causes the interval <= 0
     */
    
    public void setFrequency(int frequency)
        throws IllegalArgumentException, ArithmeticException
    {
        if(frequency <= 0)
        {
            throw new ArithmeticException();
	}
	else if (this.millis/frequency < 0 && this.nanos/frequency < 0)
        {
            throw new IllegalArgumentException();
	}
        
        this.frequency=frequency;
    }
}
